// use available at https://omacronides.com/lab/firefox-lazy-load/test.zip
/**
 * Creates a "bad" list of attributes : 'loading' AFTER 'src'
 * @param {Number} index
 * @return {{alt: string, src: string, loading: string}}
 */
const badObject = (index) => {
    return {alt: `Image ${index}`, src: `${index}.jpg`, loading: 'lazy'};
};
/**
 * Creates a "good" list of attributes : 'loading' BEFORE 'src'
 * @param {Number} index 
 * @return {{loading: string, alt: string, src: string}}
 */
const goodObject = (index) => {
    return {loading: 'lazy', alt: `Image ${index}`, src: `${index}.jpg`,};
};
/**
 * Creates an image that will push it's next sibling outside the viewport
 * @param {Object} attrs Object of attributes for the image.
 * @return {HTMLImageElement}
 */
const createImage = (attrs) => {
    // Creates an image that will push it's next sibling outside the viewport
    const img = document.createElement('img');
    img.style.display = 'block';
    img.style.marginBottom = '120vh';
    // Apply attributes by reading object entries in the defined order.
    for (const [name, value] of Object.entries(attrs)) { img.setAttribute(name, value); }
    return img;
};
/**
 * Run tests : insert images into the body and read the `complete` property.
 * Only the first image should have `complete` at true.
 * The test needs 3 lightweight images in the current directory : 1.jpg, 2.jpg, 3.jpg
 * @param {String} title 
 * @param {Function} builder 
 */
const run = (title, builder) => {
    return new Promise((resolve) => {
        const images = [];
        document.body.innerHTML = '';
        for (let i = 1; i < 4; i += 1) {
            const img = createImage(builder(i));
            document.body.appendChild(img);
            images.push(img);
        }
        console.log(`[TEST] ${title}`);
        window.setTimeout(() => {
            images.forEach((img, index) => {
                if (index === 0) {
                    console.log(`\t# Image ${img.alt} complete = true`);
                    console.assert(img.complete === true);
                } else {
                    console.log(`\t# Image ${img.alt} complete = false`);
                    console.assert(img.complete === false);
                }
            });
            resolve();
        }, 250);
    });
};

// Run the tests
run('With bad objects', badObject).then(() => run('With good objects', goodObject));
